@@ -1,8 +1,7 @@
-"""http related errors."""
+"""HTTP related errors."""
 
 from asyncio import TimeoutError
 
-
 __all__ = (
     'DisconnectedError', 'ClientDisconnectedError', 'ServerDisconnectedError',
 
@@ -39,7 +38,7 @@ class ClientError(Exception):
 
 
 class ClientHttpProcessingError(ClientError):
-    """Base class for client http processing errors."""
+    """Base class for client HTTP processing errors."""
 
 
 class ClientRequestError(ClientHttpProcessingError):
@@ -71,9 +70,9 @@ class ProxyConnectionError(ClientConnectionError):
 
 
 class HttpProcessingError(Exception):
-    """Http error.
+    """HTTP error.
 
-    Shortcut for raising http errors with custom code, message and headers.
+    Shortcut for raising HTTP errors with custom code, message and headers.
 
     :param int code: HTTP Error code.
     :param str message: (optional) Error message.
@@ -96,12 +95,9 @@ class HttpProcessingError(Exception):
 class WSServerHandshakeError(HttpProcessingError):
     """websocket server handshake error."""
 
-    def __init__(self, message, *, headers=None):
-        super().__init__(message=message, headers=headers)
-
 
 class HttpProxyError(HttpProcessingError):
-    """Http proxy error.
+    """HTTP proxy error.
 
     Raised in :class:`aiohttp.connector.ProxyConnector` if
     proxy responds with status other than ``200 OK``
@@ -148,6 +144,8 @@ class LineTooLong(BadHttpMessage):
 class InvalidHeader(BadHttpMessage):
 
     def __init__(self, hdr):
+        if isinstance(hdr, bytes):
+            hdr = hdr.decode('utf-8', 'surrogateescape')
         super().__init__('Invalid HTTP Header: {}'.format(hdr))
         self.hdr = hdr
 
@@ -161,11 +159,7 @@ class BadStatusLine(BadHttpMessage):
         self.line = line
 
 
-class ParserError(Exception):
-    """Base parser error."""
-
-
-class LineLimitExceededParserError(ParserError):
+class LineLimitExceededParserError(HttpBadRequest):
     """Line is too long."""
 
     def __init__(self, msg, limit):
