@@ -59,14 +59,23 @@ _SocketSocketTransport ->
 import asyncio
 import asyncio.streams
 import inspect
+import socket
+
 from . import errors
-from .streams import FlowControlDataQueue, EofStream
+from .streams import EofStream, FlowControlDataQueue
 
 __all__ = ('EofStream', 'StreamParser', 'StreamProtocol',
-           'ParserBuffer', 'LinesParser', 'ChunksParser')
+           'ParserBuffer', 'StreamWriter')
 
 DEFAULT_LIMIT = 2 ** 16
 
+if hasattr(socket, 'TCP_CORK'):  # pragma: no cover
+    CORK = socket.TCP_CORK
+elif hasattr(socket, 'TCP_NOPUSH'):  # pragma: no cover
+    CORK = socket.TCP_NOPUSH
+else:  # pragma: no cover
+    CORK = None
+
 
 class StreamParser:
     """StreamParser manages incoming bytes stream and protocol parsers.
@@ -80,7 +89,7 @@ class StreamParser:
     """
 
     def __init__(self, *, loop=None, buf=None,
-                 limit=DEFAULT_LIMIT, eof_exc_class=RuntimeError):
+                 limit=DEFAULT_LIMIT, eof_exc_class=RuntimeError, **kwargs):
         self._loop = loop
         self._eof = False
         self._exception = None
@@ -199,10 +208,9 @@ class StreamParser:
             return
 
         # TODO: write test
-        if hasattr(self._loop, 'is_closed'):
-            if self._loop.is_closed():
-                # TODO: log something
-                return
+        if self._loop.is_closed():
+            # TODO: log something
+            return
 
         try:
             self._parser.throw(EofStream())
@@ -224,6 +232,49 @@ class StreamWriter(asyncio.streams.StreamWriter):
         self._protocol = protocol
         self._reader = reader
         self._loop = loop
+        self._tcp_nodelay = False
+        self._tcp_cork = False
+        self._socket = transport.get_extra_info('socket')
+
+    @property
+    def tcp_nodelay(self):
+        return self._tcp_nodelay
+
+    def set_tcp_nodelay(self, value):
+        value = bool(value)
+        if self._tcp_nodelay == value:
+            return
+        self._tcp_nodelay = value
+        if self._socket is None:
+            return
+        if self._socket.family not in (socket.AF_INET, socket.AF_INET6):
+            return
+        if self._tcp_cork:
+            self._tcp_cork = False
+            if CORK is not None:  # pragma: no branch
+                self._socket.setsockopt(socket.IPPROTO_TCP, CORK, False)
+        self._socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, value)
+
+    @property
+    def tcp_cork(self):
+        return self._tcp_cork
+
+    def set_tcp_cork(self, value):
+        value = bool(value)
+        if self._tcp_cork == value:
+            return
+        self._tcp_cork = value
+        if self._socket is None:
+            return
+        if self._socket.family not in (socket.AF_INET, socket.AF_INET6):
+            return
+        if self._tcp_nodelay:
+            self._socket.setsockopt(socket.IPPROTO_TCP,
+                                    socket.TCP_NODELAY,
+                                    False)
+            self._tcp_nodelay = False
+        if CORK is not None:  # pragma: no branch
+            self._socket.setsockopt(socket.IPPROTO_TCP, CORK, value)
 
 
 class StreamProtocol(asyncio.streams.FlowControlMixin, asyncio.Protocol):
@@ -263,47 +314,58 @@ class StreamProtocol(asyncio.streams.FlowControlMixin, asyncio.Protocol):
         self.reader.feed_eof()
 
 
-class ParserBuffer(bytearray):
-    """ParserBuffer is a bytearray extension.
+class _ParserBufferHelper:
+
+    __slots__ = ('exception', 'data')
+
+    def __init__(self, exception, data):
+        self.exception = exception
+        self.data = data
+
+
+class ParserBuffer:
+    """ParserBuffer is NOT a bytearray extension anymore.
 
     ParserBuffer provides helper methods for parsers.
     """
-    __slots__ = ('_exception', '_writer')
+    __slots__ = ('_helper', '_writer', '_data')
 
     def __init__(self, *args):
-        super().__init__(*args)
-
-        self._exception = None
-        self._writer = self._feed_data()
+        self._data = bytearray(*args)
+        self._helper = _ParserBufferHelper(None, self._data)
+        self._writer = self._feed_data(self._helper)
         next(self._writer)
 
     def exception(self):
-        return self._exception
+        return self._helper.exception
 
     def set_exception(self, exc):
-        self._exception = exc
+        self._helper.exception = exc
 
-    def _feed_data(self):
+    @staticmethod
+    def _feed_data(helper):
         while True:
             chunk = yield
             if chunk:
-                self.extend(chunk)
+                helper.data.extend(chunk)
 
-            if self._exception:
-                self._writer = self._feed_data()
-                next(self._writer)
-                raise self._exception
+            if helper.exception:
+                raise helper.exception
 
     def feed_data(self, data):
-        self._writer.send(data)
+        if not self._helper.exception:
+            self._writer.send(data)
 
     def read(self, size):
         """read() reads specified amount of bytes."""
 
         while True:
-            if len(self) >= size:
-                data = self[:size]
-                del self[:size]
+            if self._helper.exception:
+                raise self._helper.exception
+
+            if len(self._data) >= size:
+                data = self._data[:size]
+                del self._data[:size]
                 return data
 
             self._writer.send((yield))
@@ -312,13 +374,16 @@ class ParserBuffer(bytearray):
         """reads size of less amount of bytes."""
 
         while True:
-            length = len(self)
+            if self._helper.exception:
+                raise self._helper.exception
+
+            length = len(self._data)
             if length > 0:
                 if size is None or length < size:
                     size = length
 
-                data = self[:size]
-                del self[:size]
+                data = self._data[:size]
+                del self._data[:size]
                 return data
 
             self._writer.send((yield))
@@ -330,7 +395,10 @@ class ParserBuffer(bytearray):
         stop_len = len(stop)
 
         while True:
-            pos = self.find(stop)
+            if self._helper.exception:
+                raise self._helper.exception
+
+            pos = self._data.find(stop)
             if pos >= 0:
                 end = pos + stop_len
                 size = end
@@ -338,11 +406,11 @@ class ParserBuffer(bytearray):
                     raise errors.LineLimitExceededParserError(
                         'Line is too long.', limit)
 
-                data = self[:size]
-                del self[:size]
+                data = self._data[:size]
+                del self._data[:size]
                 return data
             else:
-                if limit is not None and len(self) > limit:
+                if limit is not None and len(self._data) > limit:
                     raise errors.LineLimitExceededParserError(
                         'Line is too long.', limit)
 
@@ -353,8 +421,11 @@ class ParserBuffer(bytearray):
         then returns data without changing internal buffer."""
 
         while True:
-            if len(self) >= size:
-                return self[:size]
+            if self._helper.exception:
+                raise self._helper.exception
+
+            if len(self._data) >= size:
+                return self._data[:size]
 
             self._writer.send((yield))
 
@@ -366,28 +437,34 @@ class ParserBuffer(bytearray):
         stop_len = len(stop)
 
         while True:
-            pos = self.find(stop)
+            if self._helper.exception:
+                raise self._helper.exception
+
+            pos = self._data.find(stop)
             if pos >= 0:
                 size = pos + stop_len
                 if limit is not None and size > limit:
                     raise errors.LineLimitExceededParserError(
-                        'Line is too long. %s' % bytes(self), limit)
+                        'Line is too long. %s' % bytes(self._data), limit)
 
-                return self[:size]
+                return self._data[:size]
             else:
-                if limit is not None and len(self) > limit:
+                if limit is not None and len(self._data) > limit:
                     raise errors.LineLimitExceededParserError(
-                        'Line is too long. %s' % bytes(self), limit)
+                        'Line is too long. %s' % bytes(self._data), limit)
 
             self._writer.send((yield))
 
     def skip(self, size):
         """skip() skips specified amount of bytes."""
 
-        while len(self) < size:
+        while len(self._data) < size:
+            if self._helper.exception:
+                raise self._helper.exception
+
             self._writer.send((yield))
 
-        del self[:size]
+        del self._data[:size]
 
     def skipuntil(self, stop):
         """skipuntil() reads until `stop` bytes sequence."""
@@ -397,46 +474,22 @@ class ParserBuffer(bytearray):
         stop_len = len(stop)
 
         while True:
-            stop_line = self.find(stop)
+            if self._helper.exception:
+                raise self._helper.exception
+
+            stop_line = self._data.find(stop)
             if stop_line >= 0:
                 size = stop_line + stop_len
-                del self[:size]
+                del self._data[:size]
                 return
 
             self._writer.send((yield))
 
+    def extend(self, data):
+        self._data.extend(data)
 
-class LinesParser:
-    """Lines parser.
+    def __len__(self):
+        return len(self._data)
 
-    Lines parser splits a bytes stream into a chunks of data, each chunk ends
-    with \\n symbol."""
-
-    def __init__(self, limit=DEFAULT_LIMIT):
-        self._limit = limit
-
-    def __call__(self, out, buf):
-        try:
-            while True:
-                chunk = yield from buf.readuntil(b'\n', self._limit)
-                out.feed_data(chunk, len(chunk))
-        except EofStream:
-            pass
-
-
-class ChunksParser:
-    """Chunks parser.
-
-    Chunks parser splits a bytes stream into a specified
-    size chunks of data."""
-
-    def __init__(self, size=8192):
-        self._size = size
-
-    def __call__(self, out, buf):
-        try:
-            while True:
-                chunk = yield from buf.read(self._size)
-                out.feed_data(chunk, len(chunk))
-        except EofStream:
-            pass
+    def __bytes__(self):
+        return bytes(self._data)
