@@ -1,25 +1,20 @@
 import asyncio
-import warnings
 import sys
-
-
-from . import hdrs
-from . import web_reqrep
-from . import web_exceptions
-from . import web_urldispatcher
-from . import web_ws
-from .abc import AbstractRouter, AbstractMatchInfo
-from .log import web_logger
-from .protocol import HttpVersion  # noqa
-from .server import ServerHttpProtocol
-from .signals import Signal, PreSignal, PostSignal
-from .web_reqrep import *  # noqa
-from .web_exceptions import *  # noqa
-from .web_urldispatcher import *  # noqa
-from .web_ws import *  # noqa
+import warnings
 from argparse import ArgumentParser
 from importlib import import_module
 
+from . import hdrs, web_exceptions, web_reqrep, web_urldispatcher, web_ws
+from .abc import AbstractMatchInfo, AbstractRouter
+from .helpers import sentinel
+from .log import web_logger
+from .protocol import HttpVersion  # noqa
+from .server import ServerHttpProtocol
+from .signals import PostSignal, PreSignal, Signal
+from .web_exceptions import *  # noqa
+from .web_reqrep import *  # noqa
+from .web_urldispatcher import *  # noqa
+from .web_ws import *  # noqa
 
 __all__ = (web_reqrep.__all__ +
            web_exceptions.__all__ +
@@ -61,6 +56,7 @@ class RequestHandler(ServerHttpProtocol):
 
     @asyncio.coroutine
     def handle_request(self, message, payload):
+        self._manager._requests_count += 1
         if self.access_log:
             now = self._loop.time()
 
@@ -124,7 +120,12 @@ class RequestHandlerFactory:
         self._secure_proxy_ssl_header = secure_proxy_ssl_header
         self._kwargs = kwargs
         self._kwargs.setdefault('logger', app.logger)
-        self.num_connections = 0
+        self._requests_count = 0
+
+    @property
+    def requests_count(self):
+        """Number of processed requests."""
+        return self._requests_count
 
     @property
     def secure_proxy_ssl_header(self):
@@ -141,48 +142,17 @@ class RequestHandlerFactory:
         if handler in self._connections:
             del self._connections[handler]
 
-    @asyncio.coroutine
-    def _connections_cleanup(self):
-        sleep = 0.05
-        while self._connections:
-            yield from asyncio.sleep(sleep, loop=self._loop)
-            if sleep < 5:
-                sleep = sleep * 2
-
     @asyncio.coroutine
     def finish_connections(self, timeout=None):
-        # try to close connections in 90% of graceful timeout
-        timeout90 = None
-        if timeout:
-            timeout90 = timeout / 100 * 90
-
-        for handler in self._connections.keys():
-            handler.closing(timeout=timeout90)
-
-        if timeout:
-            try:
-                yield from asyncio.wait_for(
-                    self._connections_cleanup(), timeout, loop=self._loop)
-            except asyncio.TimeoutError:
-                self._app.logger.warning(
-                    "Not all connections are closed (pending: %d)",
-                    len(self._connections))
-
-        for transport in self._connections.values():
-            transport.close()
-
+        coros = [conn.shutdown(timeout) for conn in self._connections]
+        yield from asyncio.gather(*coros, loop=self._loop)
         self._connections.clear()
 
     def __call__(self):
-        self.num_connections += 1
-        try:
-            return self._handler(
-                self, self._app, self._router, loop=self._loop,
-                secure_proxy_ssl_header=self._secure_proxy_ssl_header,
-                **self._kwargs)
-        except:
-            web_logger.exception(
-                'Can not create request handler: {!r}'.format(self._handler))
+        return self._handler(
+            self, self._app, self._router, loop=self._loop,
+            secure_proxy_ssl_header=self._secure_proxy_ssl_header,
+            **self._kwargs)
 
 
 class Application(dict):
@@ -202,13 +172,12 @@ class Application(dict):
         self._loop = loop
         self.logger = logger
 
-        for factory in middlewares:
-            assert asyncio.iscoroutinefunction(factory), factory
         self._middlewares = list(middlewares)
 
         self._on_pre_signal = PreSignal()
         self._on_post_signal = PostSignal()
         self._on_response_prepare = Signal(self)
+        self._on_startup = Signal(self)
         self._on_shutdown = Signal(self)
         self._on_cleanup = Signal(self)
 
@@ -228,6 +197,10 @@ class Application(dict):
     def on_post_signal(self):
         return self._on_post_signal
 
+    @property
+    def on_startup(self):
+        return self._on_startup
+
     @property
     def on_shutdown(self):
         return self._on_shutdown
@@ -249,8 +222,31 @@ class Application(dict):
         return self._middlewares
 
     def make_handler(self, **kwargs):
-        return self._handler_factory(
-            self, self.router, loop=self.loop, **kwargs)
+        debug = kwargs.pop('debug', sentinel)
+        if debug is not sentinel:
+            warnings.warn(
+                "`debug` parameter is deprecated. "
+                "Use Application's debug mode instead", DeprecationWarning)
+            if debug != self.debug:
+                raise ValueError(
+                    "The value of `debug` parameter conflicts with the debug "
+                    "settings of the `Application` instance. The "
+                    "application's debug mode setting should be used instead "
+                    "as a single point to setup a debug mode. For more "
+                    "information please check "
+                    "http://aiohttp.readthedocs.io/en/stable/"
+                    "web_reference.html#aiohttp.web.Application"
+                )
+        return self._handler_factory(self, self.router, debug=self.debug,
+                                     loop=self.loop, **kwargs)
+
+    @asyncio.coroutine
+    def startup(self):
+        """Causes on_startup signal
+
+        Should be called in the event loop along with the request handler.
+        """
+        yield from self.on_startup.send(self)
 
     @asyncio.coroutine
     def shutdown(self):
@@ -294,7 +290,7 @@ class Application(dict):
 
 def run_app(app, *, host='0.0.0.0', port=None,
             shutdown_timeout=60.0, ssl_context=None,
-            print=print):
+            print=print, backlog=128):
     """Run an app locally"""
     if port is None:
         if not ssl_context:
@@ -305,18 +301,20 @@ def run_app(app, *, host='0.0.0.0', port=None,
     loop = app.loop
 
     handler = app.make_handler()
-    srv = loop.run_until_complete(loop.create_server(handler, host, port,
-                                                     ssl=ssl_context))
+    server = loop.create_server(handler, host, port, ssl=ssl_context,
+                                backlog=backlog)
+    srv, startup_res = loop.run_until_complete(asyncio.gather(server,
+                                                              app.startup(),
+                                                              loop=loop))
 
     scheme = 'https' if ssl_context else 'http'
-    prompt = '127.0.0.1' if host == '0.0.0.0' else host
-    print("======== Running on {scheme}://{prompt}:{port}/ ========\n"
+    print("======== Running on {scheme}://{host}:{port}/ ========\n"
           "(Press CTRL+C to quit)".format(
-              scheme=scheme, prompt=prompt, port=port))
+              scheme=scheme, host=host, port=port))
 
     try:
         loop.run_forever()
-    except KeyboardInterrupt:  # pragma: no branch
+    except KeyboardInterrupt:  # pragma: no cover
         pass
     finally:
         srv.close()
@@ -372,5 +370,5 @@ def main(argv):
     run_app(app, host=args.hostname, port=args.port)
     arg_parser.exit(message="Stopped\n")
 
-if __name__ == "__main__":
-    main(sys.argv[1:])
+if __name__ == "__main__":  # pragma: no branch
+    main(sys.argv[1:])  # pragma: no cover
