@@ -3,33 +3,29 @@ import binascii
 import cgi
 import collections
 import datetime
+import enum
 import http.cookies
 import io
 import json
 import math
 import time
 import warnings
-
-import enum
-
 from email.utils import parsedate
 from types import MappingProxyType
-from urllib.parse import urlsplit, parse_qsl, unquote
+from urllib.parse import parse_qsl, unquote, urlsplit
 
-from . import hdrs
-from .helpers import reify
-from .multidict import (CIMultiDictProxy,
-                        CIMultiDict,
-                        MultiDictProxy,
-                        MultiDict)
-from .protocol import Response as ResponseImpl, HttpVersion10, HttpVersion11
+from multidict import CIMultiDict, CIMultiDictProxy, MultiDict, MultiDictProxy
+
+from . import hdrs, multipart
+from .helpers import reify, sentinel
+from .protocol import Response as ResponseImpl
+from .protocol import HttpVersion10, HttpVersion11
 from .streams import EOF_MARKER
 
-
-__all__ = ('ContentCoding', 'Request', 'StreamResponse', 'Response')
-
-
-sentinel = object()
+__all__ = (
+    'ContentCoding', 'Request', 'StreamResponse', 'Response',
+    'json_response'
+)
 
 
 class HeadersMixin:
@@ -96,29 +92,20 @@ class Request(dict, HeadersMixin):
                     hdrs.METH_TRACE, hdrs.METH_DELETE}
 
     def __init__(self, app, message, payload, transport, reader, writer, *,
-                 _HOST=hdrs.HOST, secure_proxy_ssl_header=None):
+                 secure_proxy_ssl_header=None):
         self._app = app
-        self._version = message.version
+        self._message = message
         self._transport = transport
         self._reader = reader
         self._writer = writer
-        self._method = message.method
-        self._host = message.headers.get(_HOST)
-        self._path_qs = message.path
         self._post = None
         self._post_files_cache = None
-        self._headers = CIMultiDictProxy(message.headers)
-        if self._version < HttpVersion10:
-            self._keep_alive = False
-        else:
-            self._keep_alive = not message.should_close
 
         # matchdict, route_name, handler
         # or information about traversal lookup
         self._match_info = None  # initialized after route resolving
 
         self._payload = payload
-        self._cookies = None
 
         self._read_bytes = None
         self._has_body = not payload.at_eof()
@@ -136,48 +123,48 @@ class Request(dict, HeadersMixin):
         secure_proxy_ssl_header = self._secure_proxy_ssl_header
         if secure_proxy_ssl_header is not None:
             header, value = secure_proxy_ssl_header
-            if self._headers.get(header) == value:
+            if self.headers.get(header) == value:
                 return 'https'
         return 'http'
 
-    @property
+    @reify
     def method(self):
         """Read only property for getting HTTP method.
 
         The value is upper-cased str like 'GET', 'POST', 'PUT' etc.
         """
-        return self._method
+        return self._message.method
 
-    @property
+    @reify
     def version(self):
         """Read only property for getting HTTP version of request.
 
         Returns aiohttp.protocol.HttpVersion instance.
         """
-        return self._version
+        return self._message.version
 
-    @property
+    @reify
     def host(self):
         """Read only property for getting *HOST* header of request.
 
         Returns str or None if HTTP request has no HOST header.
         """
-        return self._host
+        return self._message.headers.get(hdrs.HOST)
 
-    @property
+    @reify
     def path_qs(self):
         """The URL including PATH_INFO and the query string.
 
         E.g, /app/blog?id=10
         """
-        return self._path_qs
+        return self._message.path
 
     @reify
     def _splitted_path(self):
-        url = '{}://{}{}'.format(self.scheme, self.host, self._path_qs)
+        url = '{}://{}{}'.format(self.scheme, self.host, self.path_qs)
         return urlsplit(url)
 
-    @property
+    @reify
     def raw_path(self):
         """ The URL including raw *PATH INFO* without the host or scheme.
         Warning, the path is unquoted and may contains non valid URL characters
@@ -221,12 +208,17 @@ class Request(dict, HeadersMixin):
             raise RuntimeError("POST is not available before post()")
         return self._post
 
-    @property
+    @reify
     def headers(self):
         """A case-insensitive multidict proxy with all headers."""
-        return self._headers
+        return CIMultiDictProxy(self._message.headers)
 
-    @property
+    @reify
+    def raw_headers(self):
+        """A sequence of pars for all headers."""
+        return tuple(self._message.raw_headers)
+
+    @reify
     def if_modified_since(self, _IF_MODIFIED_SINCE=hdrs.IF_MODIFIED_SINCE):
         """The value of If-Modified-Since HTTP header, or None.
 
@@ -240,10 +232,13 @@ class Request(dict, HeadersMixin):
                                          tzinfo=datetime.timezone.utc)
         return None
 
-    @property
+    @reify
     def keep_alive(self):
         """Is keepalive enabled by client?"""
-        return self._keep_alive
+        if self.version < HttpVersion10:
+            return False
+        else:
+            return not self._message.should_close
 
     @property
     def match_info(self):
@@ -260,24 +255,16 @@ class Request(dict, HeadersMixin):
         """Transport used for request processing."""
         return self._transport
 
-    @property
+    @reify
     def cookies(self):
         """Return request cookies.
 
         A read-only dictionary-like object.
         """
-        if self._cookies is None:
-            raw = self.headers.get(hdrs.COOKIE, '')
-            parsed = http.cookies.SimpleCookie(raw)
-            self._cookies = MappingProxyType(
-                {key: val.value for key, val in parsed.items()})
-        return self._cookies
-
-    @property
-    def payload(self):
-        """Return raw payload stream."""
-        warnings.warn('use Request.content instead', DeprecationWarning)
-        return self._payload
+        raw = self.headers.get(hdrs.COOKIE, '')
+        parsed = http.cookies.SimpleCookie(raw)
+        return MappingProxyType(
+            {key: val.value for key, val in parsed.items()})
 
     @property
     def content(self):
@@ -323,10 +310,20 @@ class Request(dict, HeadersMixin):
         return bytes_body.decode(encoding)
 
     @asyncio.coroutine
-    def json(self, *, loader=json.loads):
+    def json(self, *, loads=json.loads, loader=None):
         """Return BODY as JSON."""
+        if loader is not None:
+            warnings.warn(
+                "Using loader argument is deprecated, use loads instead",
+                DeprecationWarning)
+            loads = loader
         body = yield from self.text()
-        return loader(body)
+        return loads(body)
+
+    @asyncio.coroutine
+    def multipart(self, *, reader=multipart.MultipartReader):
+        """Return async iterator to process BODY as multipart."""
+        return reader(self.headers, self.content)
 
     @asyncio.coroutine
     def post(self):
@@ -344,6 +341,10 @@ class Request(dict, HeadersMixin):
             self._post = MultiDictProxy(MultiDict())
             return self._post
 
+        if self.content_type.startswith('multipart/'):
+            warnings.warn('To process multipart requests use .multipart'
+                          ' coroutine instead.', DeprecationWarning)
+
         body = yield from self.read()
         content_charset = self.charset or 'utf-8'
 
@@ -389,9 +390,14 @@ class Request(dict, HeadersMixin):
         self._post = MultiDictProxy(out)
         return self._post
 
+    def copy(self):
+        raise NotImplementedError
+
     def __repr__(self):
+        ascii_encodable_path = self.path.encode('ascii', 'backslashreplace') \
+            .decode('ascii')
         return "<{} {} {} >".format(self.__class__.__name__,
-                                    self.method, self.path)
+                                    self.method, ascii_encodable_path)
 
 
 ############################################################
@@ -415,9 +421,13 @@ class StreamResponse(HeadersMixin):
         self._req = None
         self._resp_impl = None
         self._eof_sent = False
+        self._tcp_nodelay = True
+        self._tcp_cork = False
 
         if headers is not None:
             self._headers.extend(headers)
+        self._parse_content_type(self._headers.get(hdrs.CONTENT_TYPE))
+        self._generate_content_type_header()
 
     def _copy_cookies(self):
         for cookie in self._cookies.values():
@@ -472,6 +482,10 @@ class StreamResponse(HeadersMixin):
         # Backwards compatibility for when force was a bool <0.17.
         if type(force) == bool:
             force = ContentCoding.deflate if force else ContentCoding.identity
+        elif force is not None:
+            assert isinstance(force, ContentCoding), ("force should one of "
+                                                      "None, bool or "
+                                                      "ContentEncoding")
 
         self._compression = True
         self._compression_force = force
@@ -500,8 +514,12 @@ class StreamResponse(HeadersMixin):
 
         self._cookies[name] = value
         c = self._cookies[name]
+
         if expires is not None:
             c['expires'] = expires
+        elif c.get('expires') == 'Thu, 01 Jan 1970 00:00:00 GMT':
+            del c['expires']
+
         if domain is not None:
             c['domain'] = domain
 
@@ -526,7 +544,9 @@ class StreamResponse(HeadersMixin):
         """
         # TODO: do we need domain/path here?
         self._cookies.pop(name, None)
-        self.set_cookie(name, '', max_age=0, domain=domain, path=path)
+        self.set_cookie(name, '', max_age=0,
+                        expires="Thu, 01 Jan 1970 00:00:00 GMT",
+                        domain=domain, path=path)
 
     @property
     def content_length(self):
@@ -587,8 +607,7 @@ class StreamResponse(HeadersMixin):
     @last_modified.setter
     def last_modified(self, value):
         if value is None:
-            if hdrs.LAST_MODIFIED in self.headers:
-                del self.headers[hdrs.LAST_MODIFIED]
+            self.headers.pop(hdrs.LAST_MODIFIED, None)
         elif isinstance(value, (int, float)):
             self.headers[hdrs.LAST_MODIFIED] = time.strftime(
                 "%a, %d %b %Y %H:%M:%S GMT", time.gmtime(math.ceil(value)))
@@ -598,6 +617,36 @@ class StreamResponse(HeadersMixin):
         elif isinstance(value, str):
             self.headers[hdrs.LAST_MODIFIED] = value
 
+    @property
+    def tcp_nodelay(self):
+        return self._tcp_nodelay
+
+    def set_tcp_nodelay(self, value):
+        value = bool(value)
+        self._tcp_nodelay = value
+        if value:
+            self._tcp_cork = False
+        if self._resp_impl is None:
+            return
+        if value:
+            self._resp_impl.transport.set_tcp_cork(False)
+        self._resp_impl.transport.set_tcp_nodelay(value)
+
+    @property
+    def tcp_cork(self):
+        return self._tcp_cork
+
+    def set_tcp_cork(self, value):
+        value = bool(value)
+        self._tcp_cork = value
+        if value:
+            self._tcp_nodelay = False
+        if self._resp_impl is None:
+            return
+        if value:
+            self._resp_impl.transport.set_tcp_nodelay(False)
+        self._resp_impl.transport.set_tcp_cork(value)
+
     def _generate_content_type_header(self, CONTENT_TYPE=hdrs.CONTENT_TYPE):
         params = '; '.join("%s=%s" % i for i in self._content_dict.items())
         if params:
@@ -610,27 +659,27 @@ class StreamResponse(HeadersMixin):
         if self._resp_impl is not None:
             if self._req is not request:
                 raise RuntimeError(
-                    'Response has been started with different request.')
+                    "Response has been started with different request.")
             else:
                 return self._resp_impl
         else:
             return None
 
-    def _start_compression(self, request):
-        def _start(coding):
-            if coding != ContentCoding.identity:
-                self.headers[hdrs.CONTENT_ENCODING] = coding.value
-                self._resp_impl.add_compression_filter(coding.value)
-                self.content_length = None
+    def _do_start_compression(self, coding):
+        if coding != ContentCoding.identity:
+            self.headers[hdrs.CONTENT_ENCODING] = coding.value
+            self._resp_impl.add_compression_filter(coding.value)
+            self.content_length = None
 
+    def _start_compression(self, request):
         if self._compression_force:
-            _start(self._compression_force)
+            self._do_start_compression(self._compression_force)
         else:
             accept_encoding = request.headers.get(
                 hdrs.ACCEPT_ENCODING, '').lower()
             for coding in ContentCoding:
                 if coding.value in accept_encoding:
-                    _start(coding)
+                    self._do_start_compression(coding)
                     return
 
     def start(self, request):
@@ -682,12 +731,20 @@ class StreamResponse(HeadersMixin):
         for key, val in headers:
             resp_impl.add_header(key, val)
 
-        resp_impl.send_headers()
+        resp_impl.transport.set_tcp_nodelay(self._tcp_nodelay)
+        resp_impl.transport.set_tcp_cork(self._tcp_cork)
+        self._send_headers(resp_impl)
         return resp_impl
 
+    def _send_headers(self, resp_impl):
+        # Durty hack required for
+        # https://github.com/KeepSafe/aiohttp/issues/1093
+        # File sender may override it
+        resp_impl.send_headers()
+
     def write(self, data):
         assert isinstance(data, (bytes, bytearray, memoryview)), \
-            'data argument must be byte-ish (%r)' % type(data)
+            "data argument must be byte-ish (%r)" % type(data)
 
         if self._eof_sent:
             raise RuntimeError("Cannot call write() after write_eof()")
@@ -727,34 +784,57 @@ class StreamResponse(HeadersMixin):
 class Response(StreamResponse):
 
     def __init__(self, *, body=None, status=200,
-                 reason=None, text=None, headers=None, content_type=None):
-        super().__init__(status=status, reason=reason, headers=headers)
-
+                 reason=None, text=None, headers=None, content_type=None,
+                 charset=None):
         if body is not None and text is not None:
-            raise ValueError("body and text are not allowed together.")
+            raise ValueError("body and text are not allowed together")
+
+        if headers is None:
+            headers = CIMultiDict()
+        elif not isinstance(headers, (CIMultiDict, CIMultiDictProxy)):
+            headers = CIMultiDict(headers)
+
+        if content_type is not None and ";" in content_type:
+            raise ValueError("charset must not be in content_type "
+                             "argument")
 
         if text is not None:
-            if hdrs.CONTENT_TYPE not in self.headers:
+            if hdrs.CONTENT_TYPE in headers:
+                if content_type or charset:
+                    raise ValueError("passing both Content-Type header and "
+                                     "content_type or charset params "
+                                     "is forbidden")
+            else:
                 # fast path for filling headers
                 if not isinstance(text, str):
-                    raise TypeError('text argument must be str (%r)' %
+                    raise TypeError("text argument must be str (%r)" %
                                     type(text))
                 if content_type is None:
                     content_type = 'text/plain'
-                self.headers[hdrs.CONTENT_TYPE] = (
-                    content_type + '; charset=utf-8')
-                self._content_type = content_type
-                self._content_dict = {'charset': 'utf-8'}
-                self.body = text.encode('utf-8')
-            else:
-                self.text = text
+                if charset is None:
+                    charset = 'utf-8'
+                headers[hdrs.CONTENT_TYPE] = (
+                    content_type + '; charset=' + charset)
+                body = text.encode(charset)
+                text = None
         else:
-            if content_type:
-                self.content_type = content_type
-            if body is not None:
-                self.body = body
+            if hdrs.CONTENT_TYPE in headers:
+                if content_type is not None or charset is not None:
+                    raise ValueError("passing both Content-Type header and "
+                                     "content_type or charset params "
+                                     "is forbidden")
             else:
-                self.body = None
+                if content_type is not None:
+                    if charset is not None:
+                        content_type += '; charset=' + charset
+                    headers[hdrs.CONTENT_TYPE] = content_type
+
+        super().__init__(status=status, reason=reason, headers=headers)
+        self.set_tcp_cork(True)
+        if text is not None:
+            self.text = text
+        else:
+            self.body = body
 
     @property
     def body(self):
@@ -763,7 +843,7 @@ class Response(StreamResponse):
     @body.setter
     def body(self, body):
         if body is not None and not isinstance(body, bytes):
-            raise TypeError('body argument must be bytes (%r)' % type(body))
+            raise TypeError("body argument must be bytes (%r)" % type(body))
         self._body = body
         if body is not None:
             self.content_length = len(body)
@@ -779,7 +859,7 @@ class Response(StreamResponse):
     @text.setter
     def text(self, text):
         if text is not None and not isinstance(text, str):
-            raise TypeError('text argument must be str (%r)' % type(text))
+            raise TypeError("text argument must be str (%r)" % type(text))
 
         if self.content_type == 'application/octet-stream':
             self.content_type = 'text/plain'
@@ -790,7 +870,26 @@ class Response(StreamResponse):
 
     @asyncio.coroutine
     def write_eof(self):
-        body = self._body
-        if body is not None:
-            self.write(body)
+        try:
+            body = self._body
+            if (body is not None and
+                    self._req.method != hdrs.METH_HEAD and
+                    self._status not in [204, 304]):
+                self.write(body)
+        finally:
+            self.set_tcp_nodelay(True)
         yield from super().write_eof()
+
+
+def json_response(data=sentinel, *, text=None, body=None, status=200,
+                  reason=None, headers=None, content_type='application/json',
+                  dumps=json.dumps):
+    if data is not sentinel:
+        if text or body:
+            raise ValueError(
+                "only one of data, text, or body should be specified"
+            )
+        else:
+            text = dumps(data)
+    return Response(text=text, body=body, status=status, reason=reason,
+                    headers=headers, content_type=content_type)
