@@ -1,28 +1,24 @@
 import abc
 import asyncio
-
-import keyword
 import collections
-import mimetypes
-import re
-import os
-import sys
 import inspect
+import keyword
+import os
+import re
+import sys
 import warnings
-
-from collections.abc import Sized, Iterable, Container
+from collections.abc import Container, Iterable, Sized
 from pathlib import Path
-from urllib.parse import urlencode, unquote
 from types import MappingProxyType
+from urllib.parse import unquote, urlencode
 
 from . import hdrs
-from .abc import AbstractRouter, AbstractMatchInfo, AbstractView
+from .abc import AbstractMatchInfo, AbstractRouter, AbstractView
+from .file_sender import FileSender
 from .protocol import HttpVersion11
-from .web_exceptions import (HTTPMethodNotAllowed, HTTPNotFound,
-                             HTTPNotModified, HTTPExpectationFailed)
-from .web_reqrep import StreamResponse
-from .multidict import upstr
-
+from .web_exceptions import (HTTPExpectationFailed, HTTPForbidden,
+                             HTTPMethodNotAllowed, HTTPNotFound)
+from .web_reqrep import Response, StreamResponse
 
 __all__ = ('UrlDispatcher', 'UrlMappingMatchInfo',
            'AbstractResource', 'Resource', 'PlainResource', 'DynamicResource',
@@ -34,6 +30,9 @@ __all__ = ('UrlDispatcher', 'UrlMappingMatchInfo',
 PY_35 = sys.version_info >= (3, 5)
 
 
+HTTP_METHOD_RE = re.compile(r"^[0-9A-Za-z!#\$%&'\*\+\-\.\^_`\|~]+$")
+
+
 class AbstractResource(Sized, Iterable):
 
     def __init__(self, *, name=None):
@@ -60,14 +59,13 @@ class AbstractResource(Sized, Iterable):
 
     @staticmethod
     def _append_query(url, query):
-        if query is not None:
+        if query:
             return url + "?" + urlencode(query)
         else:
             return url
 
 
-class AbstractRoute(metaclass=abc.ABCMeta):
-    METHODS = hdrs.METH_ALL | {hdrs.METH_ANY}
+class AbstractRoute(abc.ABC):
 
     def __init__(self, method, handler, *,
                  expect_handler=None,
@@ -79,8 +77,8 @@ class AbstractRoute(metaclass=abc.ABCMeta):
         assert asyncio.iscoroutinefunction(expect_handler), \
             'Coroutine is expected, got {!r}'.format(expect_handler)
 
-        method = upstr(method)
-        if method not in self.METHODS:
+        method = method.upper()
+        if not HTTP_METHOD_RE.match(method):
             raise ValueError("{} is not allowed HTTP method".format(method))
 
         assert callable(handler), handler
@@ -93,7 +91,14 @@ class AbstractRoute(metaclass=abc.ABCMeta):
               issubclass(handler, AbstractView)):
             pass
         else:
-            handler = asyncio.coroutine(handler)
+            @asyncio.coroutine
+            def handler_wrapper(*args, **kwargs):
+                result = old_handler(*args, **kwargs)
+                if asyncio.iscoroutine(result):
+                    result = yield from result
+                return result
+            old_handler = handler
+            handler = handler_wrapper
 
         self._method = method
         self._handler = handler
@@ -176,7 +181,7 @@ class MatchInfoError(UrlMappingMatchInfo):
 
 @asyncio.coroutine
 def _defaultExpectHandler(request):
-    """Default handler for Except header.
+    """Default handler for Expect header.
 
     Just send "100 Continue" to client.
     raise HTTPExpectationFailed if value of header is not "100-continue"
@@ -204,10 +209,11 @@ class ResourceAdapter(AbstractResource):
     @asyncio.coroutine
     def resolve(self, method, path):
         route_method = self._route.method
-        allowed_methods = {route_method}
-        if route_method == method or route_method == hdrs.METH_ANY:
-            match_dict = self._route.match(path)
-            if match_dict is not None:
+        allowed_methods = set()
+        match_dict = self._route.match(path)
+        if match_dict is not None:
+            allowed_methods.add(route_method)
+            if route_method == hdrs.METH_ANY or route_method == method:
                 return (UrlMappingMatchInfo(match_dict, self._route),
                         allowed_methods)
         return None, allowed_methods
@@ -257,11 +263,10 @@ class Resource(AbstractResource):
 
         for route in self._routes:
             route_method = route.method
+            allowed_methods.add(route_method)
 
             if route_method == method or route_method == hdrs.METH_ANY:
                 return UrlMappingMatchInfo(match_dict, route), allowed_methods
-
-            allowed_methods.add(route_method)
         else:
             return None, allowed_methods
 
@@ -432,7 +437,8 @@ class StaticRoute(Route):
 
     def __init__(self, name, prefix, directory, *,
                  expect_handler=None, chunk_size=256*1024,
-                 response_factory=StreamResponse):
+                 response_factory=StreamResponse,
+                 show_index=False):
         assert prefix.startswith('/'), prefix
         assert prefix.endswith('/'), prefix
         super().__init__(
@@ -440,18 +446,19 @@ class StaticRoute(Route):
         self._prefix = prefix
         self._prefix_len = len(self._prefix)
         try:
-            directory = Path(directory).resolve()
+            directory = Path(directory)
+            if str(directory).startswith('~'):
+                directory = Path(os.path.expanduser(str(directory)))
+            directory = directory.resolve()
             if not directory.is_dir():
                 raise ValueError('Not a directory')
         except (FileNotFoundError, ValueError) as error:
             raise ValueError(
                 "No directory exists at '{}'".format(directory)) from error
         self._directory = directory
-        self._chunk_size = chunk_size
-        self._response_factory = response_factory
-
-        if bool(os.environ.get("AIOHTTP_NOSENDFILE")):
-            self._sendfile = self._sendfile_fallback
+        self._file_sender = FileSender(resp_factory=response_factory,
+                                       chunk_size=chunk_size)
+        self._show_index = show_index
 
     def match(self, path):
         if not path.startswith(self._prefix):
@@ -470,91 +477,9 @@ class StaticRoute(Route):
         return {'directory': self._directory,
                 'prefix': self._prefix}
 
-    def _sendfile_cb(self, fut, out_fd, in_fd, offset, count, loop,
-                     registered):
-        if registered:
-            loop.remove_writer(out_fd)
-        try:
-            n = os.sendfile(out_fd, in_fd, offset, count)
-            if n == 0:  # EOF reached
-                n = count
-        except (BlockingIOError, InterruptedError):
-            n = 0
-        except Exception as exc:
-            fut.set_exception(exc)
-            return
-
-        if n < count:
-            loop.add_writer(out_fd, self._sendfile_cb, fut, out_fd, in_fd,
-                            offset + n, count - n, loop, True)
-        else:
-            fut.set_result(None)
-
-    @asyncio.coroutine
-    def _sendfile_system(self, req, resp, fobj, count):
-        """
-        Write `count` bytes of `fobj` to `resp` starting from `offset` using
-        the ``sendfile`` system call.
-
-        `req` should be a :obj:`aiohttp.web.Request` instance.
-
-        `resp` should be a :obj:`aiohttp.web.StreamResponse` instance.
-
-        `fobj` should be an open file object.
-
-        `offset` should be an integer >= 0.
-
-        `count` should be an integer > 0.
-        """
-        transport = req.transport
-
-        if transport.get_extra_info("sslcontext"):
-            yield from self._sendfile_fallback(req, resp, fobj, count)
-            return
-
-        yield from resp.drain()
-
-        loop = req.app.loop
-        out_fd = transport.get_extra_info("socket").fileno()
-        in_fd = fobj.fileno()
-        fut = asyncio.Future(loop=loop)
-
-        self._sendfile_cb(fut, out_fd, in_fd, 0, count, loop, False)
-
-        yield from fut
-
-    @asyncio.coroutine
-    def _sendfile_fallback(self, req, resp, fobj, count):
-        """
-        Mimic the :meth:`_sendfile_system` method, but without using the
-        ``sendfile`` system call. This should be used on systems that don't
-        support the ``sendfile`` system call.
-
-        To avoid blocking the event loop & to keep memory usage low, `fobj` is
-        transferred in chunks controlled by the `chunk_size` argument to
-        :class:`StaticRoute`.
-        """
-        chunk_size = self._chunk_size
-
-        chunk = fobj.read(chunk_size)
-        while chunk and count > chunk_size:
-            resp.write(chunk)
-            yield from resp.drain()
-            count = count - chunk_size
-            chunk = fobj.read(chunk_size)
-
-        if chunk:
-            resp.write(chunk[:count])
-            yield from resp.drain()
-
-    if hasattr(os, "sendfile"):  # pragma: no cover
-        _sendfile = _sendfile_system
-    else:  # pragma: no cover
-        _sendfile = _sendfile_fallback
-
     @asyncio.coroutine
     def handle(self, request):
-        filename = request.match_info['filename']
+        filename = unquote(request.match_info['filename'])
         try:
             filepath = self._directory.joinpath(filename).resolve()
             filepath.relative_to(self._directory)
@@ -563,39 +488,61 @@ class StaticRoute(Route):
             raise HTTPNotFound() from error
         except Exception as error:
             # perm error or other kind!
-            request.logger.exception(error)
+            request.app.logger.exception(error)
             raise HTTPNotFound() from error
 
-        st = filepath.stat()
+        # on opening a dir, load it's contents if allowed
+        if filepath.is_dir():
+            if self._show_index:
+                try:
+                    ret = Response(text=self._directory_as_html(filepath))
+                except PermissionError:
+                    raise HTTPForbidden()
+            else:
+                raise HTTPForbidden()
+        elif filepath.is_file():
+            ret = yield from self._file_sender.send(request, filepath)
+        else:
+            raise HTTPNotFound
 
-        modsince = request.if_modified_since
-        if modsince is not None and st.st_mtime <= modsince.timestamp():
-            raise HTTPNotModified()
+        return ret
 
-        ct, encoding = mimetypes.guess_type(str(filepath))
-        if not ct:
-            ct = 'application/octet-stream'
+    def _directory_as_html(self, filepath):
+        "returns directory's index as html"
+        # sanity check
+        assert filepath.is_dir()
 
-        resp = self._response_factory()
-        resp.content_type = ct
-        if encoding:
-            resp.headers[hdrs.CONTENT_ENCODING] = encoding
-        resp.last_modified = st.st_mtime
+        posix_dir_len = len(self._directory.as_posix())
 
-        file_size = st.st_size
+        # remove the beginning of posix path, so it would be relative
+        # to our added static path
+        relative_path_to_dir = filepath.as_posix()[posix_dir_len:]
+        index_of = "Index of /{}".format(relative_path_to_dir)
+        head = "<head>\n<title>{}</title>\n</head>".format(index_of)
+        h1 = "<h1>{}</h1>".format(index_of)
 
-        resp.content_length = file_size
-        resp.set_tcp_cork(True)
-        try:
-            yield from resp.prepare(request)
+        index_list = []
+        dir_index = filepath.iterdir()
+        for _file in sorted(dir_index):
+            # show file url as relative to static path
+            file_url = _file.as_posix()[posix_dir_len:]
 
-            with filepath.open('rb') as f:
-                yield from self._sendfile(request, resp, f, file_size)
+            # if file is a directory, add '/' to the end of the name
+            if _file.is_dir():
+                file_name = "{}/".format(_file.name)
+            else:
+                file_name = _file.name
 
-        finally:
-            resp.set_tcp_nodelay(True)
+            index_list.append(
+                '<li><a href="{url}">{name}</a></li>'.format(url=file_url,
+                                                             name=file_name)
+            )
+        ul = "<ul>\n{}\n</ul>".format('\n'.join(index_list))
+        body = "<body>\n{}\n{}\n</body>".format(h1, ul)
 
-        return resp
+        html = "<html>\n{}\n{}\n</html>".format(head, body)
+
+        return html
 
     def __repr__(self):
         name = "'" + self.name + "' " if self.name is not None else ""
@@ -652,7 +599,8 @@ class View(AbstractView):
             return (yield from self.__iter__())
 
     def _raise_allowed_methods(self):
-        allowed_methods = {m for m in hdrs.METH_ALL if hasattr(self, m)}
+        allowed_methods = {
+            m for m in hdrs.METH_ALL if hasattr(self, m.lower())}
         raise HTTPMethodNotAllowed(self.request.method, allowed_methods)
 
 
@@ -820,11 +768,13 @@ class UrlDispatcher(AbstractRouter, collections.abc.Mapping):
                                   expect_handler=expect_handler)
 
     def add_static(self, prefix, path, *, name=None, expect_handler=None,
-                   chunk_size=256*1024, response_factory=StreamResponse):
-        """
-        Adds static files view
-        :param prefix - url prefix
-        :param path - folder with files
+                   chunk_size=256*1024, response_factory=StreamResponse,
+                   show_index=False):
+        """Add static files view.
+
+        prefix - url prefix
+        path - folder with files
+
         """
         assert prefix.startswith('/')
         if not prefix.endswith('/'):
@@ -832,6 +782,43 @@ class UrlDispatcher(AbstractRouter, collections.abc.Mapping):
         route = StaticRoute(name, prefix, path,
                             expect_handler=expect_handler,
                             chunk_size=chunk_size,
-                            response_factory=response_factory)
+                            response_factory=response_factory,
+                            show_index=show_index)
         self.register_route(route)
         return route
+
+    def add_head(self, *args, **kwargs):
+        """
+        Shortcut for add_route with method HEAD
+        """
+        return self.add_route(hdrs.METH_HEAD, *args, **kwargs)
+
+    def add_get(self, *args, **kwargs):
+        """
+        Shortcut for add_route with method GET
+        """
+        return self.add_route(hdrs.METH_GET, *args, **kwargs)
+
+    def add_post(self, *args, **kwargs):
+        """
+        Shortcut for add_route with method POST
+        """
+        return self.add_route(hdrs.METH_POST, *args, **kwargs)
+
+    def add_put(self, *args, **kwargs):
+        """
+        Shortcut for add_route with method PUT
+        """
+        return self.add_route(hdrs.METH_PUT, *args, **kwargs)
+
+    def add_patch(self, *args, **kwargs):
+        """
+        Shortcut for add_route with method PATCH
+        """
+        return self.add_route(hdrs.METH_PATCH, *args, **kwargs)
+
+    def add_delete(self, *args, **kwargs):
+        """
+        Shortcut for add_route with method DELETE
+        """
+        return self.add_route(hdrs.METH_DELETE, *args, **kwargs)
