@@ -2,22 +2,22 @@
 
 import asyncio
 import gc
-import unittest
-import unittest.mock
-
 import inspect
 import io
-import urllib.parse
 import os.path
-
+import re
+import unittest
+import urllib.parse
+import zlib
 from http.cookies import SimpleCookie
+from unittest import mock
 
 import pytest
+from multidict import CIMultiDict, CIMultiDictProxy, upstr
 
 import aiohttp
+from aiohttp import BaseConnector, helpers
 from aiohttp.client_reqrep import ClientRequest, ClientResponse
-from aiohttp.multidict import upstr, CIMultiDict, CIMultiDictProxy
-from aiohttp import BaseConnector
 
 
 @pytest.yield_fixture
@@ -265,13 +265,10 @@ def test_basic_auth_utf8(make_request):
     assert 'Basic bmtpbTrRgdC10LrRgNC10YI=' == req.headers['AUTHORIZATION']
 
 
-def test_basic_auth_tuple_deprecated(make_request, warning):
-    with warning(DeprecationWarning):
-        req = make_request('get', 'http://python.org',
-                           auth=('nkim', '1234'))
-
-        assert 'AUTHORIZATION' in req.headers
-        assert 'Basic bmtpbToxMjM0' == req.headers['AUTHORIZATION']
+def test_basic_auth_tuple_forbidden(make_request):
+    with pytest.raises(TypeError):
+        make_request('get', 'http://python.org',
+                     auth=('nkim', '1234'))
 
 
 def test_basic_auth_from_url(make_request):
@@ -312,13 +309,23 @@ def test_path_safe_chars_preserved(make_request):
 def test_params_are_added_before_fragment1(make_request):
     req = make_request('GET', "http://example.com/path#fragment",
                        params={"a": "b"})
-    assert req.path == "/path?a=b#fragment"
+    assert req.url == "http://example.com/path?a=b#fragment"
 
 
 def test_params_are_added_before_fragment2(make_request):
     req = make_request('GET', "http://example.com/path?key=value#fragment",
                        params={"a": "b"})
-    assert req.path == "/path?key=value&a=b#fragment"
+    assert req.url == "http://example.com/path?key=value&a=b#fragment"
+
+
+def test_path_not_contain_fragment1(make_request):
+    req = make_request('GET', "http://example.com/path#fragment")
+    assert req.path == "/path"
+
+
+def test_path_not_contain_fragment2(make_request):
+    req = make_request('GET', "http://example.com/path?key=value#fragment")
+    assert req.path == "/path?key=value"
 
 
 def test_cookies(make_request):
@@ -372,6 +379,25 @@ def test_query_multivalued_param(make_request):
         assert req.path == '/?test=foo&test=baz'
 
 
+def test_query_str_param(make_request):
+    for meth in ClientRequest.ALL_METHODS:
+        req = make_request(meth, 'http://python.org', params='test=foo')
+        assert req.path == '/?test=foo'
+
+
+def test_query_bytes_param_raises(make_request):
+    for meth in ClientRequest.ALL_METHODS:
+        with pytest.raises(TypeError) as ctx:
+            make_request(meth, 'http://python.org', params=b'test=foo')
+        assert re.match('not a valid non-string.*or mapping', str(ctx.value))
+
+
+def test_query_str_param_is_not_encoded(make_request):
+    for meth in ClientRequest.ALL_METHODS:
+        req = make_request(meth, 'http://python.org', params='test=f+oo')
+        assert req.path == '/?test=f+oo'
+
+
 def test_params_update_path_and_url(make_request):
     req = make_request('get', 'http://python.org',
                        params=(('test', 'foo'), ('test', 'baz')))
@@ -379,15 +405,51 @@ def test_params_update_path_and_url(make_request):
     assert req.url == 'http://python.org/?test=foo&test=baz'
 
 
+def test_params_empty_path_and_url(make_request):
+    req_empty = make_request('get', 'http://python.org', params={})
+    assert req_empty.path == '/'
+    assert req_empty.url == 'http://python.org/'
+    req_none = make_request('get', 'http://python.org')
+    assert req_none.path == '/'
+    assert req_none.url == 'http://python.org/'
+
+
+def test_gen_netloc_all(make_request):
+    req = make_request('get',
+                       'https://aiohttp:pwpwpw@' +
+                       '12345678901234567890123456789' +
+                       '012345678901234567890:8080')
+    assert req.netloc == '12345678901234567890123456789' +\
+                         '012345678901234567890:8080'
+
+
+def test_gen_netloc_no_port(make_request):
+    req = make_request('get',
+                       'https://aiohttp:pwpwpw@' +
+                       '12345678901234567890123456789' +
+                       '012345678901234567890/')
+    assert req.netloc == '12345678901234567890123456789' +\
+                         '012345678901234567890'
+
+
+def test_gen_notloc_failed(make_request):
+    with pytest.raises(ValueError) as excinfo:
+        make_request('get',
+                     'https://aiohttp:pwpwpw@' +
+                     '123456789012345678901234567890123456789' +
+                     '01234567890123456789012345/')
+        assert excinfo.value.message == "URL has an invalid label."
+
+
 class TestClientRequest(unittest.TestCase):
 
     def setUp(self):
         self.loop = asyncio.new_event_loop()
         asyncio.set_event_loop(None)
 
-        self.transport = unittest.mock.Mock()
-        self.connection = unittest.mock.Mock()
-        self.protocol = unittest.mock.Mock()
+        self.transport = mock.Mock()
+        self.connection = mock.Mock()
+        self.protocol = mock.Mock()
         self.protocol.writer.drain.return_value = ()
         self.stream = aiohttp.StreamParser(loop=self.loop)
         self.connector = BaseConnector(loop=self.loop)
@@ -440,15 +502,15 @@ class TestClientRequest(unittest.TestCase):
 
     def test_content_type_skip_auto_header_bytes(self):
         req = ClientRequest('post', 'http://python.org', data=b'hey you',
-                            skip_auto_headers=set('CONTENT-TYPE'),
+                            skip_auto_headers={'Content-Type'},
                             loop=self.loop)
         resp = req.send(self.transport, self.protocol)
-        self.assertNotIn('application/octet-stream', req.headers)
+        self.assertNotIn('CONTENT-TYPE', req.headers)
         resp.close()
 
     def test_content_type_skip_auto_header_form(self):
         req = ClientRequest('post', 'http://python.org', data={'hey': 'you'},
-                            loop=self.loop, skip_auto_headers={'CONTENT-TYPE'})
+                            loop=self.loop, skip_auto_headers={'Content-Type'})
         resp = req.send(self.transport, self.protocol)
         self.assertNotIn('CONTENT-TYPE', req.headers)
         resp.close()
@@ -456,7 +518,7 @@ class TestClientRequest(unittest.TestCase):
     def test_content_type_auto_header_content_length_no_skip(self):
         req = ClientRequest('get', 'http://python.org',
                             data=io.BytesIO(b'hey'),
-                            skip_auto_headers={'CONTENT-LENGTH'},
+                            skip_auto_headers={'Content-Length'},
                             loop=self.loop)
         resp = req.send(self.transport, self.protocol)
         self.assertEqual(req.headers.get('CONTENT-LENGTH'), '3')
@@ -475,7 +537,7 @@ class TestClientRequest(unittest.TestCase):
             self.loop.run_until_complete(req.close())
             resp.close()
 
-    @unittest.mock.patch(
+    @mock.patch(
         'aiohttp.client_reqrep.ClientRequest.update_body_from_data')
     def test_pass_falsy_data(self, _):
         req = ClientRequest(
@@ -506,9 +568,9 @@ class TestClientRequest(unittest.TestCase):
             self.loop.run_until_complete(req.close())
             resp.close()
 
-    @unittest.mock.patch('aiohttp.client_reqrep.aiohttp')
+    @mock.patch('aiohttp.client_reqrep.aiohttp')
     def test_content_encoding(self, m_http):
-        req = ClientRequest('get', 'http://python.org/',
+        req = ClientRequest('get', 'http://python.org/', data='foo',
                             compress='deflate', loop=self.loop)
         resp = req.send(self.transport, self.protocol)
         self.assertEqual(req.headers['TRANSFER-ENCODING'], 'chunked')
@@ -518,10 +580,20 @@ class TestClientRequest(unittest.TestCase):
         self.loop.run_until_complete(req.close())
         resp.close()
 
-    @unittest.mock.patch('aiohttp.client_reqrep.aiohttp')
+    @mock.patch('aiohttp.client_reqrep.aiohttp')
+    def test_content_encoding_dont_set_headers_if_no_body(self, m_http):
+        req = ClientRequest('get', 'http://python.org/',
+                            compress='deflate', loop=self.loop)
+        resp = req.send(self.transport, self.protocol)
+        self.assertNotIn('TRANSFER-ENCODING', req.headers)
+        self.assertNotIn('CONTENT-ENCODING', req.headers)
+        self.loop.run_until_complete(req.close())
+        resp.close()
+
+    @mock.patch('aiohttp.client_reqrep.aiohttp')
     def test_content_encoding_header(self, m_http):
         req = ClientRequest(
-            'get', 'http://python.org/',
+            'get', 'http://python.org/', data='foo',
             headers={'Content-Encoding': 'deflate'}, loop=self.loop)
         resp = req.send(self.transport, self.protocol)
         self.assertEqual(req.headers['TRANSFER-ENCODING'], 'chunked')
@@ -551,7 +623,7 @@ class TestClientRequest(unittest.TestCase):
         self.loop.run_until_complete(req.close())
         resp.close()
 
-    @unittest.mock.patch('aiohttp.client_reqrep.aiohttp')
+    @mock.patch('aiohttp.client_reqrep.aiohttp')
     def test_chunked_explicit(self, m_http):
         req = ClientRequest(
             'get', 'http://python.org/', chunked=True, loop=self.loop)
@@ -563,7 +635,7 @@ class TestClientRequest(unittest.TestCase):
         self.loop.run_until_complete(req.close())
         resp.close()
 
-    @unittest.mock.patch('aiohttp.client_reqrep.aiohttp')
+    @mock.patch('aiohttp.client_reqrep.aiohttp')
     def test_chunked_explicit_size(self, m_http):
         req = ClientRequest(
             'get', 'http://python.org/', chunked=1024, loop=self.loop)
@@ -597,6 +669,20 @@ class TestClientRequest(unittest.TestCase):
                              str(os.path.getsize(fname)))
             self.loop.run_until_complete(req.close())
 
+    def test_precompressed_data_stays_intact(self):
+        data = zlib.compress(b'foobar')
+        req = ClientRequest(
+            'post', 'http://python.org/',
+            data=data,
+            headers={'CONTENT-ENCODING': 'deflate'},
+            compress=False,
+            loop=self.loop)
+        self.assertFalse(req.compress)
+        self.assertFalse(req.chunked)
+        self.assertEqual(req.headers['CONTENT-ENCODING'],
+                         'deflate')
+        self.loop.run_until_complete(req.close())
+
     def test_file_upload_not_chunked_seek(self):
         here = os.path.dirname(__file__)
         fname = os.path.join(here, 'sample.key')
@@ -657,10 +743,9 @@ class TestClientRequest(unittest.TestCase):
         self.loop.run_until_complete(resp.wait_for_close())
         self.assertIsNone(req._writer)
         self.assertEqual(
-            self.transport.write.mock_calls[-3:],
-            [unittest.mock.call(b'binary data result'),
-             unittest.mock.call(b'\r\n'),
-             unittest.mock.call(b'0\r\n\r\n')])
+            self.transport.write.mock_calls[-2:],
+            [mock.call(b'12\r\nbinary data result\r\n'),
+             mock.call(b'0\r\n\r\n')])
         self.loop.run_until_complete(req.close())
 
     def test_data_file(self):
@@ -677,14 +762,13 @@ class TestClientRequest(unittest.TestCase):
         self.loop.run_until_complete(resp.wait_for_close())
         self.assertIsNone(req._writer)
         self.assertEqual(
-            self.transport.write.mock_calls[-3:],
-            [unittest.mock.call(b'*' * 2),
-             unittest.mock.call(b'\r\n'),
-             unittest.mock.call(b'0\r\n\r\n')])
+            self.transport.write.mock_calls[-2:],
+            [mock.call(b'2\r\n' + b'*' * 2 + b'\r\n'),
+             mock.call(b'0\r\n\r\n')])
         self.loop.run_until_complete(req.close())
 
     def test_data_stream_exc(self):
-        fut = asyncio.Future(loop=self.loop)
+        fut = helpers.create_future(self.loop)
 
         def gen():
             yield b'binary data'
@@ -724,7 +808,7 @@ class TestClientRequest(unittest.TestCase):
         resp.close()
 
     def test_data_stream_exc_chain(self):
-        fut = asyncio.Future(loop=self.loop)
+        fut = helpers.create_future(self.loop)
 
         def gen():
             yield from fut
@@ -772,10 +856,9 @@ class TestClientRequest(unittest.TestCase):
         resp = req.send(self.transport, self.protocol)
         self.loop.run_until_complete(req._writer)
         self.assertEqual(
-            self.transport.write.mock_calls[-3:],
-            [unittest.mock.call(b'binary data result'),
-             unittest.mock.call(b'\r\n'),
-             unittest.mock.call(b'0\r\n\r\n')])
+            self.transport.write.mock_calls[-2:],
+            [mock.call(b'12\r\nbinary data result\r\n'),
+             mock.call(b'0\r\n\r\n')])
         self.loop.run_until_complete(req.close())
         resp.close()
 
@@ -796,7 +879,7 @@ class TestClientRequest(unittest.TestCase):
         self.loop.run_until_complete(req._writer)
         self.assertEqual(
             self.transport.write.mock_calls[-1],
-            unittest.mock.call(b'data'))
+            mock.call(b'data'))
         self.loop.run_until_complete(req.close())
         resp.close()
 
@@ -811,10 +894,9 @@ class TestClientRequest(unittest.TestCase):
         resp = req.send(self.transport, self.protocol)
         self.loop.run_until_complete(req.close())
         self.assertEqual(
-            self.transport.write.mock_calls[-3:],
-            [unittest.mock.call(b'result'),
-             unittest.mock.call(b'\r\n'),
-             unittest.mock.call(b'0\r\n\r\n')])
+            self.transport.write.mock_calls[-2:],
+            [mock.call(b'6\r\nresult\r\n'),
+             mock.call(b'0\r\n\r\n')])
         self.loop.run_until_complete(req.close())
         resp.close()
 
@@ -835,7 +917,7 @@ class TestClientRequest(unittest.TestCase):
         req = ClientRequest('get', 'http://python.org', loop=self.loop)
         resp = req.send(self.transport, self.protocol)
         self.assertIsNotNone(req._writer)
-        writer = req._writer = unittest.mock.Mock()
+        writer = req._writer = mock.Mock()
 
         req.terminate()
         self.assertIsNone(req._writer)
@@ -843,12 +925,10 @@ class TestClientRequest(unittest.TestCase):
         resp.close()
 
     def test_terminate_with_closed_loop(self):
-        if not hasattr(self.loop, 'is_closed'):
-            self.skipTest("Required asyncio 3.4.2+")
         req = ClientRequest('get', 'http://python.org', loop=self.loop)
         resp = req.send(self.transport, self.protocol)
         self.assertIsNotNone(req._writer)
-        writer = req._writer = unittest.mock.Mock()
+        writer = req._writer = mock.Mock()
 
         self.loop.close()
         req.terminate()
