@@ -4,9 +4,10 @@
 import asyncio
 import gc
 import unittest
-import unittest.mock
+from unittest import mock
 
 import aiohttp
+from aiohttp import helpers
 from aiohttp.client_reqrep import ClientResponse
 
 
@@ -16,7 +17,7 @@ class TestClientResponse(unittest.TestCase):
         self.loop = asyncio.new_event_loop()
         asyncio.set_event_loop(None)
 
-        self.connection = unittest.mock.Mock()
+        self.connection = mock.Mock()
         self.stream = aiohttp.StreamParser(loop=self.loop)
         self.response = ClientResponse('get', 'http://def-cl-resp.org')
         self.response._post_init(self.loop)
@@ -31,7 +32,7 @@ class TestClientResponse(unittest.TestCase):
         response = ClientResponse('get', 'http://del-cl-resp.org')
         response._post_init(self.loop)
 
-        connection = unittest.mock.Mock()
+        connection = mock.Mock()
         response._setup_connection(connection)
         self.loop.set_exception_handler(lambda loop, ctx: None)
 
@@ -69,12 +70,25 @@ class TestClientResponse(unittest.TestCase):
             '<ClientResponse(http://def-cl-resp.org) [200 Ok]>',
             repr(self.response))
 
+    def test_repr_non_ascii_url(self):
+        response = ClientResponse('get', 'http://fake-host.org/\u03bb')
+        self.assertIn(
+            "<ClientResponse(http://fake-host.org/\\u03bb) [None None]>",
+            repr(response))
+
+    def test_repr_non_ascii_reason(self):
+        response = ClientResponse('get', 'http://fake-host.org/path')
+        response.reason = '\u03bb'
+        self.assertIn(
+            "<ClientResponse(http://fake-host.org/path) [None \\u03bb]>",
+            repr(response))
+
     def test_read_and_release_connection(self):
         def side_effect(*args, **kwargs):
-            fut = asyncio.Future(loop=self.loop)
+            fut = helpers.create_future(self.loop)
             fut.set_result(b'payload')
             return fut
-        content = self.response.content = unittest.mock.Mock()
+        content = self.response.content = mock.Mock()
         content.read.side_effect = side_effect
 
         res = self.loop.run_until_complete(self.response.read())
@@ -82,8 +96,8 @@ class TestClientResponse(unittest.TestCase):
         self.assertIsNone(self.response._connection)
 
     def test_read_and_release_connection_with_error(self):
-        content = self.response.content = unittest.mock.Mock()
-        content.read.return_value = asyncio.Future(loop=self.loop)
+        content = self.response.content = mock.Mock()
+        content.read.return_value = helpers.create_future(self.loop)
         content.read.return_value.set_exception(ValueError)
 
         self.assertRaises(
@@ -92,43 +106,22 @@ class TestClientResponse(unittest.TestCase):
         self.assertTrue(self.response._closed)
 
     def test_release(self):
-        fut = asyncio.Future(loop=self.loop)
+        fut = helpers.create_future(self.loop)
         fut.set_result(b'')
-        content = self.response.content = unittest.mock.Mock()
+        content = self.response.content = mock.Mock()
         content.readany.return_value = fut
 
         self.loop.run_until_complete(self.response.release())
         self.assertIsNone(self.response._connection)
 
-    def test_read_and_close(self):
-        self.response.read = unittest.mock.Mock()
-        self.response.read.return_value = asyncio.Future(loop=self.loop)
-        self.response.read.return_value.set_result(b'data')
-
-        with self.assertWarns(DeprecationWarning):
-            res = self.loop.run_until_complete(self.response.read_and_close())
-        self.assertEqual(res, b'data')
-        self.assertTrue(self.response.read.called)
-
-    def test_read_decode_deprecated(self):
-        self.response._content = b'data'
-        self.response.json = unittest.mock.Mock()
-        self.response.json.return_value = asyncio.Future(loop=self.loop)
-        self.response.json.return_value.set_result('json')
-
-        with self.assertWarns(DeprecationWarning):
-            res = self.loop.run_until_complete(self.response.read(decode=True))
-        self.assertEqual(res, 'json')
-        self.assertTrue(self.response.json.called)
-
     def test_text(self):
         def side_effect(*args, **kwargs):
-            fut = asyncio.Future(loop=self.loop)
+            fut = helpers.create_future(self.loop)
             fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
             return fut
         self.response.headers = {
-            'CONTENT-TYPE': 'application/json;charset=cp1251'}
-        content = self.response.content = unittest.mock.Mock()
+            'Content-Type': 'application/json;charset=cp1251'}
+        content = self.response.content = mock.Mock()
         content.read.side_effect = side_effect
 
         res = self.loop.run_until_complete(self.response.text())
@@ -137,14 +130,14 @@ class TestClientResponse(unittest.TestCase):
 
     def test_text_custom_encoding(self):
         def side_effect(*args, **kwargs):
-            fut = asyncio.Future(loop=self.loop)
+            fut = helpers.create_future(self.loop)
             fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
             return fut
         self.response.headers = {
-            'CONTENT-TYPE': 'application/json'}
-        content = self.response.content = unittest.mock.Mock()
+            'Content-Type': 'application/json'}
+        content = self.response.content = mock.Mock()
         content.read.side_effect = side_effect
-        self.response._get_encoding = unittest.mock.Mock()
+        self.response._get_encoding = mock.Mock()
 
         res = self.loop.run_until_complete(
             self.response.text(encoding='cp1251'))
@@ -154,11 +147,11 @@ class TestClientResponse(unittest.TestCase):
 
     def test_text_detect_encoding(self):
         def side_effect(*args, **kwargs):
-            fut = asyncio.Future(loop=self.loop)
+            fut = helpers.create_future(self.loop)
             fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
             return fut
-        self.response.headers = {'CONTENT-TYPE': 'application/json'}
-        content = self.response.content = unittest.mock.Mock()
+        self.response.headers = {'Content-Type': 'application/json'}
+        content = self.response.content = mock.Mock()
         content.read.side_effect = side_effect
 
         self.loop.run_until_complete(self.response.read())
@@ -168,12 +161,12 @@ class TestClientResponse(unittest.TestCase):
 
     def test_text_after_read(self):
         def side_effect(*args, **kwargs):
-            fut = asyncio.Future(loop=self.loop)
+            fut = helpers.create_future(self.loop)
             fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
             return fut
         self.response.headers = {
-            'CONTENT-TYPE': 'application/json;charset=cp1251'}
-        content = self.response.content = unittest.mock.Mock()
+            'Content-Type': 'application/json;charset=cp1251'}
+        content = self.response.content = mock.Mock()
         content.read.side_effect = side_effect
 
         res = self.loop.run_until_complete(self.response.text())
@@ -182,12 +175,12 @@ class TestClientResponse(unittest.TestCase):
 
     def test_json(self):
         def side_effect(*args, **kwargs):
-            fut = asyncio.Future(loop=self.loop)
+            fut = helpers.create_future(self.loop)
             fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
             return fut
         self.response.headers = {
-            'CONTENT-TYPE': 'application/json;charset=cp1251'}
-        content = self.response.content = unittest.mock.Mock()
+            'Content-Type': 'application/json;charset=cp1251'}
+        content = self.response.content = mock.Mock()
         content.read.side_effect = side_effect
 
         res = self.loop.run_until_complete(self.response.json())
@@ -196,7 +189,7 @@ class TestClientResponse(unittest.TestCase):
 
     def test_json_custom_loader(self):
         self.response.headers = {
-            'CONTENT-TYPE': 'application/json;charset=cp1251'}
+            'Content-Type': 'application/json;charset=cp1251'}
         self.response._content = b'data'
 
         def custom(content):
@@ -205,10 +198,10 @@ class TestClientResponse(unittest.TestCase):
         res = self.loop.run_until_complete(self.response.json(loads=custom))
         self.assertEqual(res, 'data-custom')
 
-    @unittest.mock.patch('aiohttp.client_reqrep.client_logger')
+    @mock.patch('aiohttp.client_reqrep.client_logger')
     def test_json_no_content(self, m_log):
         self.response.headers = {
-            'CONTENT-TYPE': 'data/octet-stream'}
+            'Content-Type': 'data/octet-stream'}
         self.response._content = b''
 
         res = self.loop.run_until_complete(self.response.json())
@@ -219,14 +212,14 @@ class TestClientResponse(unittest.TestCase):
 
     def test_json_override_encoding(self):
         def side_effect(*args, **kwargs):
-            fut = asyncio.Future(loop=self.loop)
+            fut = helpers.create_future(self.loop)
             fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
             return fut
         self.response.headers = {
-            'CONTENT-TYPE': 'application/json;charset=utf8'}
-        content = self.response.content = unittest.mock.Mock()
+            'Content-Type': 'application/json;charset=utf8'}
+        content = self.response.content = mock.Mock()
         content.read.side_effect = side_effect
-        self.response._get_encoding = unittest.mock.Mock()
+        self.response._get_encoding = mock.Mock()
 
         res = self.loop.run_until_complete(
             self.response.json(encoding='cp1251'))
@@ -236,11 +229,11 @@ class TestClientResponse(unittest.TestCase):
 
     def test_json_detect_encoding(self):
         def side_effect(*args, **kwargs):
-            fut = asyncio.Future(loop=self.loop)
+            fut = helpers.create_future(self.loop)
             fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
             return fut
-        self.response.headers = {'CONTENT-TYPE': 'application/json'}
-        content = self.response.content = unittest.mock.Mock()
+        self.response.headers = {'Content-Type': 'application/json'}
+        content = self.response.content = mock.Mock()
         content.read.side_effect = side_effect
 
         res = self.loop.run_until_complete(self.response.json())
@@ -249,22 +242,29 @@ class TestClientResponse(unittest.TestCase):
 
     def test_override_flow_control(self):
         class MyResponse(ClientResponse):
-            flow_control_class = aiohttp.FlowControlDataQueue
+            flow_control_class = aiohttp.StreamReader
         response = MyResponse('get', 'http://my-cl-resp.org')
         response._post_init(self.loop)
         response._setup_connection(self.connection)
-        self.assertIsInstance(response.content, aiohttp.FlowControlDataQueue)
+        self.assertIsInstance(response.content, aiohttp.StreamReader)
         response.close()
 
-    @unittest.mock.patch('aiohttp.client_reqrep.chardet')
+    @mock.patch('aiohttp.client_reqrep.chardet')
     def test_get_encoding_unknown(self, m_chardet):
         m_chardet.detect.return_value = {'encoding': None}
 
-        self.response.headers = {'CONTENT-TYPE': 'application/json'}
+        self.response.headers = {'Content-Type': 'application/json'}
         self.assertEqual(self.response._get_encoding(), 'utf-8')
 
-    def test_close_deprecated(self):
-        self.response._connection = self.connection
-        with self.assertWarns(DeprecationWarning):
-            self.response.close(force=False)
-        self.assertIsNone(self.response._connection)
+    def test_raise_for_status_2xx(self):
+        self.response.status = 200
+        self.response.reason = 'OK'
+        self.response.raise_for_status()  # should not raise
+
+    def test_raise_for_status_4xx(self):
+        self.response.status = 409
+        self.response.reason = 'CONFLICT'
+        with self.assertRaises(aiohttp.HttpProcessingError) as cm:
+            self.response.raise_for_status()
+        self.assertEqual(str(cm.exception.code), '409')
+        self.assertEqual(str(cm.exception.message), "CONFLICT")
