@@ -1,15 +1,13 @@
 """Tests for http/wsgi.py"""
 
-import io
 import asyncio
+import io
 import socket
 import unittest
-import unittest.mock
+from unittest import mock
 
 import aiohttp
-from aiohttp import multidict
-from aiohttp import wsgi
-from aiohttp import protocol
+from aiohttp import helpers, multidict, protocol, wsgi
 
 
 class TestHttpWsgiServerProtocol(unittest.TestCase):
@@ -18,13 +16,13 @@ class TestHttpWsgiServerProtocol(unittest.TestCase):
         self.loop = asyncio.new_event_loop()
         asyncio.set_event_loop(None)
 
-        self.wsgi = unittest.mock.Mock()
-        self.reader = unittest.mock.Mock()
-        self.writer = unittest.mock.Mock()
+        self.wsgi = mock.Mock()
+        self.reader = mock.Mock()
+        self.writer = mock.Mock()
         self.writer.drain.return_value = ()
-        self.transport = unittest.mock.Mock()
+        self.transport = mock.Mock()
         self.transport.get_extra_info.side_effect = [
-            unittest.mock.Mock(family=socket.AF_INET),
+            mock.Mock(family=socket.AF_INET),
             ('1.2.3.4', 1234),
             ('2.3.4.5', 80)]
 
@@ -83,8 +81,8 @@ class TestHttpWsgiServerProtocol(unittest.TestCase):
         self.assertEqual(environ['SERVER_PORT'], '80')
         get_extra_info_calls = self.transport.get_extra_info.mock_calls
         expected_calls = [
-            unittest.mock.call('socket'),
-            unittest.mock.call('peername'),
+            mock.call('socket'),
+            mock.call('peername'),
         ]
         self.assertEqual(expected_calls, get_extra_info_calls)
 
@@ -129,7 +127,7 @@ class TestHttpWsgiServerProtocol(unittest.TestCase):
             resp.start_response,
             '500 Err', [('CONTENT-TYPE', 'text/plain')], ['', ValueError()])
 
-    @unittest.mock.patch('aiohttp.wsgi.aiohttp')
+    @mock.patch('aiohttp.wsgi.aiohttp')
     def test_wsgi_response_101_upgrade_to_websocket(self, m_asyncio):
         srv = self._make_srv()
         resp = srv.create_wsgi_response(self.message)
@@ -156,9 +154,9 @@ class TestHttpWsgiServerProtocol(unittest.TestCase):
 
         def wsgi_app(env, start):
             start('200 OK', [('Content-Type', 'text/plain')])
-            f1 = asyncio.Future(loop=self.loop)
+            f1 = helpers.create_future(self.loop)
             f1.set_result(b'data')
-            fut = asyncio.Future(loop=self.loop)
+            fut = helpers.create_future(self.loop)
             fut.set_result([f1])
             return fut
 
@@ -193,7 +191,7 @@ class TestHttpWsgiServerProtocol(unittest.TestCase):
             [c[1][0] for c in self.writer.write.mock_calls])
         self.assertTrue(content.startswith(b'HTTP/1.1 200 OK'))
         self.assertTrue(content.endswith(b'data\r\n0\r\n\r\n'))
-        self.assertFalse(srv._keep_alive)
+        self.assertFalse(srv._keepalive)
 
     def test_handle_request_io(self):
 
@@ -234,7 +232,7 @@ class TestHttpWsgiServerProtocol(unittest.TestCase):
             [c[1][0] for c in self.writer.write.mock_calls])
         self.assertTrue(content.startswith(b'HTTP/1.1 200 OK'))
         self.assertTrue(content.endswith(b'data\r\n0\r\n\r\n'))
-        self.assertTrue(srv._keep_alive)
+        self.assertTrue(srv._keepalive)
 
     def test_handle_request_readpayload(self):
 
@@ -281,7 +279,7 @@ class TestHttpWsgiServerProtocol(unittest.TestCase):
 
     def test_family_inet6(self):
         self.transport.get_extra_info.side_effect = [
-            unittest.mock.Mock(family=socket.AF_INET6),
+            mock.Mock(family=socket.AF_INET6),
             ("::", 1122, 0, 0),
             ('2.3.4.5', 80)]
         self.message = protocol.RawRequestMessage(
@@ -297,7 +295,7 @@ class TestHttpWsgiServerProtocol(unittest.TestCase):
         if not hasattr(socket, "AF_UNIX"):
             self.skipTest("No UNIX address family. (Windows?)")
         self.transport.get_extra_info.side_effect = [
-            unittest.mock.Mock(family=socket.AF_UNIX)]
+            mock.Mock(family=socket.AF_UNIX)]
         headers = multidict.MultiDict({
             'SERVER_NAME': '1.2.3.4', 'SERVER_PORT': '5678',
             'REMOTE_ADDR': '4.3.2.1', 'REMOTE_PORT': '8765'})
